'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var O = require('fp-ts/Option');
var TE = require('fp-ts/TaskEither');
var _function = require('fp-ts/lib/function');
var RD = require('@devexperts/remote-data-ts');
var fpTsReactStableHooks = require('fp-ts-react-stable-hooks');
var S = require('fp-ts/string');
var E = require('fp-ts/Eq');
var TO = require('fp-ts/TaskOption');

/**
 * The context that contains the Supabase Client
 * @see {@link Provider} for usage
 */
var SupabaseContext = /*#__PURE__*/react.createContext(null);
/**
 * A provider for {@link SupabaseContext}. Everything must be wrapped in this.
 * @example
 * ```ts
 * import { createClient } from '@supabase/supabase-js';
 *
 * const client = createClient('url', 'anonKey');
 *
 * return (
 *   <Provider value={client}>
 *     <Consumer />
 *   </Provider>
 * );
 * ```
 */
var Provider = SupabaseContext.Provider;

/**
 * Gets the current supabase client.
 * @returns A `SupabaseClient`, if it exists
 */
var useSupabase = function useSupabase() {
  var client = react.useContext(SupabaseContext);
  return O.fromNullable(client);
};

var queryToTE = function queryToTE(_ref) {
  var data = _ref.data,
    error = _ref.error;
  return error ? TE.left(error.message + " - " + error.details + " - " + error.hint) :
  // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
  TE.right(data);
};
var promiseLikeToPromise = function promiseLikeToPromise(promiseLike) {
  return Promise.resolve(promiseLike);
};
var promiseLikeToTask = function promiseLikeToTask(promiseLike) {
  return function () {
    return promiseLikeToPromise(promiseLike);
  };
};

/**
 * Gets many rows from a supabase table.
 * @example
 * ```tsx
 * const filter = useFilter<Foo>((query) => query.gt("id", "5"));
 * const [result, reexecute] = useTable<Foo>("foo", "id, bar", filter);
 * pipe(
 *   result,
 *   RD.fold3(
 *     () => <div>Loading...</div>,
 *     (e) => <div>Error {e}</div>,
 *     (rows) => (
 *       <div>
 *         {rows.map((row) => (
 *           <div key={row.id}>{row.bar}</div>
 *         ))}
 *       </div>
 *     )
 *   )
 * );
 * ```
 * @param tableName - The table name to get rows from
 * @param selectArgs - Arguments for a select query
 * @param filter - A filter for your query
 * @param eq - An Eq for your data type
 * @returns Rows that match your filter, or all rows if there is no filter, and a function to reexecute the query
 */
var useTable = function useTable(tableName, selectArgs, filter, eq) {
  if (selectArgs === void 0) {
    selectArgs = '*';
  }
  if (eq === void 0) {
    eq = E.eqStrict;
  }
  var supabase = useSupabase();
  var _useStable = fpTsReactStableHooks.useStable(RD.pending, RD.getEq(S.Eq, eq)),
    result = _useStable[0],
    setResult = _useStable[1];
  var execute = react.useCallback(function () {
    return _function.pipe(supabase, TE.fromOption(_function.constant('You must use useTable with a Provider!')), TE.map(function (supabase) {
      return supabase.from(tableName).select(selectArgs);
    }), TE.map(filter || _function.identity), TE.chainTaskK(promiseLikeToTask), TE.chain(queryToTE))().then(_function.flow(RD.fromEither, setResult));
  }, [supabase, filter, tableName, selectArgs]);
  react.useEffect(function () {
    execute();
  }, []);
  return [result, execute];
};

/**
 * Gets a single row from a supabase table.
 *
 * @example
 * ```ts
 * const filter = useFilter<Foo>((query) => query.gt("id", "5"));
 * const [result, execute] = useSingle<Foo>("foo", "*", filter);
 *
 * pipe(
 *   result,
 *   RD.fold3(
 *     () => <div>Loading...</div>,
 *     (e) => <div>Error {e}</div>,
 *     (row) => <div>{row.bar}</div>
 *   )
 * );
 * ```
 *
 * @param tableName - The table name to get a row from
 * @param selectArgs - Arguments for a select query
 * @param filter - A filter for your query
 * @param eq - An Eq for your data type
 * @returns A single row, and a function to reexecute the query
 */
var useSingle = function useSingle(tableName, selectArgs, filter, eq) {
  if (selectArgs === void 0) {
    selectArgs = '*';
  }
  if (eq === void 0) {
    eq = E.eqStrict;
  }
  var supabase = useSupabase();
  var _useStable = fpTsReactStableHooks.useStable(RD.pending, RD.getEq(S.Eq, eq)),
    result = _useStable[0],
    setResult = _useStable[1];
  var execute = react.useCallback(function () {
    return _function.pipe(supabase, TE.fromOption(_function.constant('You must use useSingle with a Provider!')), TE.map(function (supabase) {
      return supabase.from(tableName).select(selectArgs).limit(1);
    }), TE.map(filter || _function.identity), TE.map(function (x) {
      return x.single();
    }), TE.chainTaskK(promiseLikeToTask), TE.chain(queryToTE))().then(_function.flow(RD.fromEither, setResult));
  }, [supabase, tableName, selectArgs, filter]);
  react.useEffect(function () {
    execute();
  }, []);
  return [result, execute];
};

function _regeneratorRuntime() {
  _regeneratorRuntime = function () {
    return exports;
  };
  var exports = {},
    Op = Object.prototype,
    hasOwn = Op.hasOwnProperty,
    defineProperty = Object.defineProperty || function (obj, key, desc) {
      obj[key] = desc.value;
    },
    $Symbol = "function" == typeof Symbol ? Symbol : {},
    iteratorSymbol = $Symbol.iterator || "@@iterator",
    asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator",
    toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";
  function define(obj, key, value) {
    return Object.defineProperty(obj, key, {
      value: value,
      enumerable: !0,
      configurable: !0,
      writable: !0
    }), obj[key];
  }
  try {
    define({}, "");
  } catch (err) {
    define = function (obj, key, value) {
      return obj[key] = value;
    };
  }
  function wrap(innerFn, outerFn, self, tryLocsList) {
    var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator,
      generator = Object.create(protoGenerator.prototype),
      context = new Context(tryLocsList || []);
    return defineProperty(generator, "_invoke", {
      value: makeInvokeMethod(innerFn, self, context)
    }), generator;
  }
  function tryCatch(fn, obj, arg) {
    try {
      return {
        type: "normal",
        arg: fn.call(obj, arg)
      };
    } catch (err) {
      return {
        type: "throw",
        arg: err
      };
    }
  }
  exports.wrap = wrap;
  var ContinueSentinel = {};
  function Generator() {}
  function GeneratorFunction() {}
  function GeneratorFunctionPrototype() {}
  var IteratorPrototype = {};
  define(IteratorPrototype, iteratorSymbol, function () {
    return this;
  });
  var getProto = Object.getPrototypeOf,
    NativeIteratorPrototype = getProto && getProto(getProto(values([])));
  NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol) && (IteratorPrototype = NativeIteratorPrototype);
  var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
  function defineIteratorMethods(prototype) {
    ["next", "throw", "return"].forEach(function (method) {
      define(prototype, method, function (arg) {
        return this._invoke(method, arg);
      });
    });
  }
  function AsyncIterator(generator, PromiseImpl) {
    function invoke(method, arg, resolve, reject) {
      var record = tryCatch(generator[method], generator, arg);
      if ("throw" !== record.type) {
        var result = record.arg,
          value = result.value;
        return value && "object" == typeof value && hasOwn.call(value, "__await") ? PromiseImpl.resolve(value.__await).then(function (value) {
          invoke("next", value, resolve, reject);
        }, function (err) {
          invoke("throw", err, resolve, reject);
        }) : PromiseImpl.resolve(value).then(function (unwrapped) {
          result.value = unwrapped, resolve(result);
        }, function (error) {
          return invoke("throw", error, resolve, reject);
        });
      }
      reject(record.arg);
    }
    var previousPromise;
    defineProperty(this, "_invoke", {
      value: function (method, arg) {
        function callInvokeWithMethodAndArg() {
          return new PromiseImpl(function (resolve, reject) {
            invoke(method, arg, resolve, reject);
          });
        }
        return previousPromise = previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
      }
    });
  }
  function makeInvokeMethod(innerFn, self, context) {
    var state = "suspendedStart";
    return function (method, arg) {
      if ("executing" === state) throw new Error("Generator is already running");
      if ("completed" === state) {
        if ("throw" === method) throw arg;
        return doneResult();
      }
      for (context.method = method, context.arg = arg;;) {
        var delegate = context.delegate;
        if (delegate) {
          var delegateResult = maybeInvokeDelegate(delegate, context);
          if (delegateResult) {
            if (delegateResult === ContinueSentinel) continue;
            return delegateResult;
          }
        }
        if ("next" === context.method) context.sent = context._sent = context.arg;else if ("throw" === context.method) {
          if ("suspendedStart" === state) throw state = "completed", context.arg;
          context.dispatchException(context.arg);
        } else "return" === context.method && context.abrupt("return", context.arg);
        state = "executing";
        var record = tryCatch(innerFn, self, context);
        if ("normal" === record.type) {
          if (state = context.done ? "completed" : "suspendedYield", record.arg === ContinueSentinel) continue;
          return {
            value: record.arg,
            done: context.done
          };
        }
        "throw" === record.type && (state = "completed", context.method = "throw", context.arg = record.arg);
      }
    };
  }
  function maybeInvokeDelegate(delegate, context) {
    var method = delegate.iterator[context.method];
    if (undefined === method) {
      if (context.delegate = null, "throw" === context.method) {
        if (delegate.iterator.return && (context.method = "return", context.arg = undefined, maybeInvokeDelegate(delegate, context), "throw" === context.method)) return ContinueSentinel;
        context.method = "throw", context.arg = new TypeError("The iterator does not provide a 'throw' method");
      }
      return ContinueSentinel;
    }
    var record = tryCatch(method, delegate.iterator, context.arg);
    if ("throw" === record.type) return context.method = "throw", context.arg = record.arg, context.delegate = null, ContinueSentinel;
    var info = record.arg;
    return info ? info.done ? (context[delegate.resultName] = info.value, context.next = delegate.nextLoc, "return" !== context.method && (context.method = "next", context.arg = undefined), context.delegate = null, ContinueSentinel) : info : (context.method = "throw", context.arg = new TypeError("iterator result is not an object"), context.delegate = null, ContinueSentinel);
  }
  function pushTryEntry(locs) {
    var entry = {
      tryLoc: locs[0]
    };
    1 in locs && (entry.catchLoc = locs[1]), 2 in locs && (entry.finallyLoc = locs[2], entry.afterLoc = locs[3]), this.tryEntries.push(entry);
  }
  function resetTryEntry(entry) {
    var record = entry.completion || {};
    record.type = "normal", delete record.arg, entry.completion = record;
  }
  function Context(tryLocsList) {
    this.tryEntries = [{
      tryLoc: "root"
    }], tryLocsList.forEach(pushTryEntry, this), this.reset(!0);
  }
  function values(iterable) {
    if (iterable) {
      var iteratorMethod = iterable[iteratorSymbol];
      if (iteratorMethod) return iteratorMethod.call(iterable);
      if ("function" == typeof iterable.next) return iterable;
      if (!isNaN(iterable.length)) {
        var i = -1,
          next = function next() {
            for (; ++i < iterable.length;) if (hasOwn.call(iterable, i)) return next.value = iterable[i], next.done = !1, next;
            return next.value = undefined, next.done = !0, next;
          };
        return next.next = next;
      }
    }
    return {
      next: doneResult
    };
  }
  function doneResult() {
    return {
      value: undefined,
      done: !0
    };
  }
  return GeneratorFunction.prototype = GeneratorFunctionPrototype, defineProperty(Gp, "constructor", {
    value: GeneratorFunctionPrototype,
    configurable: !0
  }), defineProperty(GeneratorFunctionPrototype, "constructor", {
    value: GeneratorFunction,
    configurable: !0
  }), GeneratorFunction.displayName = define(GeneratorFunctionPrototype, toStringTagSymbol, "GeneratorFunction"), exports.isGeneratorFunction = function (genFun) {
    var ctor = "function" == typeof genFun && genFun.constructor;
    return !!ctor && (ctor === GeneratorFunction || "GeneratorFunction" === (ctor.displayName || ctor.name));
  }, exports.mark = function (genFun) {
    return Object.setPrototypeOf ? Object.setPrototypeOf(genFun, GeneratorFunctionPrototype) : (genFun.__proto__ = GeneratorFunctionPrototype, define(genFun, toStringTagSymbol, "GeneratorFunction")), genFun.prototype = Object.create(Gp), genFun;
  }, exports.awrap = function (arg) {
    return {
      __await: arg
    };
  }, defineIteratorMethods(AsyncIterator.prototype), define(AsyncIterator.prototype, asyncIteratorSymbol, function () {
    return this;
  }), exports.AsyncIterator = AsyncIterator, exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
    void 0 === PromiseImpl && (PromiseImpl = Promise);
    var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
    return exports.isGeneratorFunction(outerFn) ? iter : iter.next().then(function (result) {
      return result.done ? result.value : iter.next();
    });
  }, defineIteratorMethods(Gp), define(Gp, toStringTagSymbol, "Generator"), define(Gp, iteratorSymbol, function () {
    return this;
  }), define(Gp, "toString", function () {
    return "[object Generator]";
  }), exports.keys = function (val) {
    var object = Object(val),
      keys = [];
    for (var key in object) keys.push(key);
    return keys.reverse(), function next() {
      for (; keys.length;) {
        var key = keys.pop();
        if (key in object) return next.value = key, next.done = !1, next;
      }
      return next.done = !0, next;
    };
  }, exports.values = values, Context.prototype = {
    constructor: Context,
    reset: function (skipTempReset) {
      if (this.prev = 0, this.next = 0, this.sent = this._sent = undefined, this.done = !1, this.delegate = null, this.method = "next", this.arg = undefined, this.tryEntries.forEach(resetTryEntry), !skipTempReset) for (var name in this) "t" === name.charAt(0) && hasOwn.call(this, name) && !isNaN(+name.slice(1)) && (this[name] = undefined);
    },
    stop: function () {
      this.done = !0;
      var rootRecord = this.tryEntries[0].completion;
      if ("throw" === rootRecord.type) throw rootRecord.arg;
      return this.rval;
    },
    dispatchException: function (exception) {
      if (this.done) throw exception;
      var context = this;
      function handle(loc, caught) {
        return record.type = "throw", record.arg = exception, context.next = loc, caught && (context.method = "next", context.arg = undefined), !!caught;
      }
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i],
          record = entry.completion;
        if ("root" === entry.tryLoc) return handle("end");
        if (entry.tryLoc <= this.prev) {
          var hasCatch = hasOwn.call(entry, "catchLoc"),
            hasFinally = hasOwn.call(entry, "finallyLoc");
          if (hasCatch && hasFinally) {
            if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
            if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
          } else if (hasCatch) {
            if (this.prev < entry.catchLoc) return handle(entry.catchLoc, !0);
          } else {
            if (!hasFinally) throw new Error("try statement without catch or finally");
            if (this.prev < entry.finallyLoc) return handle(entry.finallyLoc);
          }
        }
      }
    },
    abrupt: function (type, arg) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
          var finallyEntry = entry;
          break;
        }
      }
      finallyEntry && ("break" === type || "continue" === type) && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc && (finallyEntry = null);
      var record = finallyEntry ? finallyEntry.completion : {};
      return record.type = type, record.arg = arg, finallyEntry ? (this.method = "next", this.next = finallyEntry.finallyLoc, ContinueSentinel) : this.complete(record);
    },
    complete: function (record, afterLoc) {
      if ("throw" === record.type) throw record.arg;
      return "break" === record.type || "continue" === record.type ? this.next = record.arg : "return" === record.type ? (this.rval = this.arg = record.arg, this.method = "return", this.next = "end") : "normal" === record.type && afterLoc && (this.next = afterLoc), ContinueSentinel;
    },
    finish: function (finallyLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.finallyLoc === finallyLoc) return this.complete(entry.completion, entry.afterLoc), resetTryEntry(entry), ContinueSentinel;
      }
    },
    catch: function (tryLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc === tryLoc) {
          var record = entry.completion;
          if ("throw" === record.type) {
            var thrown = record.arg;
            resetTryEntry(entry);
          }
          return thrown;
        }
      }
      throw new Error("illegal catch attempt");
    },
    delegateYield: function (iterable, resultName, nextLoc) {
      return this.delegate = {
        iterator: values(iterable),
        resultName: resultName,
        nextLoc: nextLoc
      }, "next" === this.method && (this.arg = undefined), ContinueSentinel;
    }
  }, exports;
}
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }
  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}
function _asyncToGenerator(fn) {
  return function () {
    var self = this,
      args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);
      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }
      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }
      _next(undefined);
    });
  };
}

/**
 * Upserts into a supabase table.
 * @example
 * ```tsx
 * const filter = useFilter<Foo>((query) => query.eq("id", "1"));
 * const [result, execute] = useUpsert<Foo>("foo");
 *
 * pipe(
 *   result,
 *   RD.fold(
 *     () => <button onClick={() => execute({ bar: "baz" }, filter)}></button>,
 *     () => <div>Loading...</div>,
 *     (e) => <div>Error {e}</div>,
 *     (rows) => (
 *       <div>
 *         Updated
 *         {rows.map((row) => (
 *           <div key={row.id}>
 *             {row.id} - {row.bar}
 *           </div>
 *         ))}
 *       </div>
 *     )
 *   )
 * );
 * ```
 * @param tableName - The table name to upsert into
 * @returns The rows that were updated
 */
var useUpsert = function useUpsert(tableName) {
  var supabase = useSupabase();
  var _useState = react.useState(RD.initial),
    result = _useState[0],
    setResult = _useState[1];
  var execute = /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(
    // eslint-disable-next-line functional/prefer-readonly-type
    values, filter) {
      return _regeneratorRuntime().wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              setResult(RD.pending);
              _function.pipe(supabase, TE.fromOption(_function.constant('You must use useUpsert from inside a Provider!')), TE.map(function (supabase) {
                return supabase.from(tableName).upsert(values);
              }), TE.map(filter || _function.identity), TE.chainTaskK(promiseLikeToTask), TE.chain(queryToTE))().then(_function.flow(RD.fromEither, setResult));
            case 2:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));
    return function execute(_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }();
  return [result, execute];
};

/**
 *  Creates a react-safe filter for use in other hooks.
 *
 * @example
 * ```ts
 * const filter = useFilter<Foo>(query => query.eq('id', 1));
 * ```
 *
 * @param filter - A function that filters your query
 * @param deps - An array of deps to regenerate your `filter`
 * @returns A filter
 */
var useFilter = function useFilter(filter, deps) {
  if (deps === void 0) {
    deps = [];
  }
  var callback = react.useCallback(filter, deps);
  return callback;
};

/**
 * Deletes data from a supabase table.
 * @example
 * ```tsx
 * const filter = useFilter<Foo>((query) => query.gt("id", "5"));
 * const [result, execute] = useDelete<Foo>("foo");
 *
 * pipe(
 *   result,
 *   RD.fold(
 *     () => <button onClick={() => execute(filter)}>Click me</button>,
 *     () => <div>Loading...</div>,
 *     (e) => <div>Error {e}</div>,
 *     (rows) => (
 *       <div>
 *         Deleted
 *         {rows.map((row) => (
 *           <div key={row.id}>{row.bar}</div>
 *         ))}
 *       </div>
 *     )
 *   )
 * );
 * ```
 * @param tableName - Name of the table you want to delete from
 * @param eq - Eq to compare your type
 * @returns A list of rows that were deleted, and the function to delete them
 */
var useDelete = function useDelete(tableName, eq) {
  if (eq === void 0) {
    eq = E.eqStrict;
  }
  var supabase = useSupabase();
  var _useStable = fpTsReactStableHooks.useStable(RD.initial, RD.getEq(S.Eq, eq)),
    result = _useStable[0],
    setResult = _useStable[1];
  var execute = /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(filter) {
      return _regeneratorRuntime().wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              setResult(RD.pending);
              _function.pipe(supabase, TE.fromOption(_function.constant('You must use useDelete from inside a Provider!')), TE.map(function (supabase) {
                return supabase.from(tableName)["delete"]();
              }), TE.map(filter), TE.chainTaskK(promiseLikeToTask), TE.chain(queryToTE))().then(_function.flow(RD.fromEither, setResult));
            case 2:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));
    return function execute(_x) {
      return _ref.apply(this, arguments);
    };
  }();
  return [result, execute];
};

/**
 * Inserts data into a supabase table.
 * @example
 * ```tsx
 * const [result, execute] = useInsert<Foo>("foo");
 *
 * pipe(
 *   result,
 *   RD.fold(
 *     () => (
 *       <button
 *         onClick={() =>
 *           execute({
 *             bar: "baz",
 *           })
 *         }
 *       >
 *         Click me
 *       </button>
 *     ),
 *     () => <div>Loading...</div>,
 *     (e) => <div>Error {e}</div>,
 *     (rows) => (
 *       <div>
 *         Inserted
 *         {rows.map((row) => (
 *           <div key={row.id}>{row.bar}</div>
 *         ))}
 *       </div>
 *     )
 *   )
 * );
 * ```
 * @param tableName - The name of the table you want to insert into
 * @param eq - An Eq for your data type
 * @returns The result of the request and a function to execute the insertion
 */
var useInsert = function useInsert(tableName, eq) {
  if (eq === void 0) {
    eq = E.eqStrict;
  }
  var supabase = useSupabase();
  var _useStable = fpTsReactStableHooks.useStable(RD.initial, RD.getEq(S.Eq, eq)),
    result = _useStable[0],
    setResult = _useStable[1];
  var execute = function execute(
  // eslint-disable-next-line functional/prefer-readonly-type
  values) {
    setResult(RD.pending);
    return _function.pipe(supabase, TE.fromOption(_function.constant('You must use useInsert from inside a Provider!')), TE.map(function (supabase) {
      return supabase.from(tableName).insert(values);
    }), TE.chainTaskK(promiseLikeToTask), TE.chain(queryToTE))().then(_function.flow(RD.fromEither, setResult));
  };
  return [result, execute];
};

/**
 * Updates some rows from a supabase table.
 * @example
 * ```tsx
 * const filter = useFilter<Foo>((query) => query.eq("id", "1"));
 * const [result, execute] = useUpdate<Foo>("foo");
 *
 * pipe(
 *   result,
 *   RD.fold(
 *     () => <button onClick={() => execute({ bar: "baz" }, filter)}></button>,
 *     () => <div>Loading...</div>,
 *     (e) => <div>Error {e}</div>,
 *     (rows) => (
 *       <div>
 *         Updated
 *         {rows.map((row) => (
 *           <div key={row.id}>
 *             {row.id} - {row.bar}
 *           </div>
 *         ))}
 *       </div>
 *     )
 *   )
 * );
 * ```
 * @param tableName - The table name to update inside of
 * @param eq - An Eq for your data type
 * @returns The rows that were updated, and a function to execute the update.
 */
var useUpdate = function useUpdate(tableName, eq) {
  if (eq === void 0) {
    eq = E.eqStrict;
  }
  var supabase = useSupabase();
  var _useStable = fpTsReactStableHooks.useStable(RD.initial, RD.getEq(S.Eq, eq)),
    result = _useStable[0],
    setResult = _useStable[1];
  var execute = /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(values, filter) {
      return _regeneratorRuntime().wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              setResult(RD.pending);
              _function.pipe(supabase, TE.fromOption(_function.constant('You must use useUpdate from inside a Provider!')), TE.map(function (supabase) {
                return supabase.from(tableName).update(values);
              }), TE.map(filter), TE.chainTaskK(promiseLikeToTask), TE.chain(queryToTE))().then(_function.flow(RD.fromEither, setResult));
            case 2:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));
    return function execute(_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }();
  return [result, execute];
};

var useAuthStateChange = function useAuthStateChange(callback) {
  var supabase = useSupabase();
  react.useEffect(function () {
    var listener = _function.pipe(supabase, O.map(function (s) {
      return s.auth.onAuthStateChange(callback);
    }));
    return function () {
      _function.pipe(listener, O.map(function (x) {
        var _x$data;
        return (_x$data = x.data) == null ? void 0 : _x$data.unsubscribe();
      }));
    };
  }, [supabase]);
};
/**
 * Returns the current logged in user, or `None` if no one is logged in
 */
var useUser = function useUser() {
  var supabase = useSupabase();
  var _useState = react.useState(O.none),
    user = _useState[0],
    setUser = _useState[1];
  react.useEffect(function () {
    return _function.pipe(supabase, O.chainNullableK(function (s) {
      return s.auth.session();
    }), O.chainNullableK(function (x) {
      return x.user;
    }), setUser);
  }, []);
  useAuthStateChange(function (_, session) {
    return _function.pipe(session == null ? void 0 : session.user, O.fromNullable, setUser);
  });
  return user;
};
/**
 * Returns `true` if a user is logged in
 */
var useUserLoggedIn = function useUserLoggedIn() {
  var user = useUser();
  return _function.pipe(user, O.isSome);
};

/**
 * Signs in a user.
 * @returns The result of the signing in, and a function to sign a user in.
 * @example
 * ```ts
 *   const [result, execute] = useSignIn();

 *   return (
 *     <div>
 *       <button
 *         onClick={e => {
 *           e.preventDefault();
 *           execute({ provider: 'google' });
 *         }}
 *       >
 *         Log in
 *       </button>
 *     </div>
 *   );
 * ```
 */
var useSignIn = function useSignIn() {
  var supabase = useSupabase();
  var _useState = react.useState(RD.initial),
    result = _useState[0],
    setResult = _useState[1];
  var execute = react.useCallback( /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(credentials, options) {
      return _regeneratorRuntime().wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              setResult(RD.pending);
              _context.next = 3;
              return _function.pipe(supabase, TO.fromOption, TO.chainTaskK(function (s) {
                return function () {
                  return s.auth.signIn(credentials, options);
                };
              }), TO.match(function () {
                setResult(RD.failure(new Error('No client found')));
              }, function (x) {
                if (x.error) {
                  setResult(RD.failure(Error(x.error.message)));
                } else {
                  _function.pipe(x.session, O.fromNullable, function (x) {
                    return RD.fromOption(x, function () {
                      return new Error('Did not return a session.');
                    });
                  }, setResult);
                }
              }))();
            case 3:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));
    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }(), [supabase]);
  return [result, execute];
};

/**
 * Signs out a user.
 * @returns The result of the signing out, and a function to sign a user out.
 * @example
 * ```ts
 * const [result, signOut] = useSignOut();
 *
 * return (
 *   <button
 *     onClick={(e) => {
 *       e.preventDefault();
 *       signOut();
 *     }}
 *   >
 *     Sign out
 *   </button>
 * );
 *
 * ```
 */
var useSignOut = function useSignOut() {
  var supabase = useSupabase();
  var _useState = react.useState(RD.initial),
    result = _useState[0],
    setResult = _useState[1];
  var execute = react.useCallback(function () {
    return _function.pipe(supabase, TO.fromOption, TO.chainTaskK(function (s) {
      return s.auth.signOut;
    }), TO.match(function () {
      return setResult(RD.failure(new Error('Supabase client not found')));
    }, function (_ref) {
      var error = _ref.error;
      if (error) {
        setResult(RD.failure(Error(error.message)));
      } else {
        setResult(RD.success(undefined));
      }
    }))();
  }, [supabase]);
  return [result, execute];
};

exports.Provider = Provider;
exports.SupabaseContext = SupabaseContext;
exports.useDelete = useDelete;
exports.useFilter = useFilter;
exports.useInsert = useInsert;
exports.useSignIn = useSignIn;
exports.useSignOut = useSignOut;
exports.useSingle = useSingle;
exports.useSupabase = useSupabase;
exports.useTable = useTable;
exports.useUpdate = useUpdate;
exports.useUpsert = useUpsert;
exports.useUser = useUser;
exports.useUserLoggedIn = useUserLoggedIn;
//# sourceMappingURL=react-supabase-fp.cjs.development.js.map
